"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

require("core-js/modules/es.string.replace");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BabelCompiler = void 0;

function _asyncToGenerator2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

  _asyncToGenerator2 = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function babel() {
  const data = _interopRequireWildcard(require("@babel/core"));

  babel = function () {
    return data;
  };

  return data;
}

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class BabelCompiler {
  constructor(id, logger, compiler, options) {
    this.id = id;
    this.logger = logger;
    this.compiler = compiler;
    this.options = options;
    (0, _defineProperty2().default)(this, "distDir", void 0);
    (0, _defineProperty2().default)(this, "distGlobPatterns", void 0);
    (0, _defineProperty2().default)(this, "shouldCopyNonSupportedFiles", void 0);
    (0, _defineProperty2().default)(this, "artifactName", void 0);
    this.distDir = options.distDir || 'dist';
    this.distGlobPatterns = options.distGlobPatterns || [`${this.distDir}/**`, `!${this.distDir}/tsconfig.tsbuildinfo`];
    this.shouldCopyNonSupportedFiles = typeof options.shouldCopyNonSupportedFiles === 'boolean' ? options.shouldCopyNonSupportedFiles : true;
    this.artifactName = options.artifactName || 'dist';
  }
  /**
   * compile one file on the workspace
   */


  transpileFile(fileContent, options) {
    const supportedExtensions = ['.ts', '.tsx', '.js', '.jsx'];

    const fileExtension = _path().default.extname(options.filePath);

    if (!supportedExtensions.includes(fileExtension) || options.filePath.endsWith('.d.ts')) {
      return null; // file is not supported
    }

    const transformOptions = this.options.babelTransformOptions || {}; // the `sourceRoot` and `sourceFileName` are manually set because the dists are written into the
    // node_modules dir, so the debugger needs to know where to find the source.

    transformOptions.sourceRoot = options.componentDir;
    transformOptions.sourceFileName = options.filePath;
    transformOptions.filename = options.filePath;
    const result = babel().transformSync(fileContent, this.options.babelTransformOptions);

    if (!result) {
      return null;
    }

    const code = result.code || '';
    const outputPath = this.replaceFileExtToJs(options.filePath);
    const mapFilePath = `${outputPath}.map`;
    const outputText = result.map ? `${code}\n\n//# sourceMappingURL=${mapFilePath}` : code;
    const outputFiles = [{
      outputText,
      outputPath
    }];

    if (result.map) {
      outputFiles.push({
        outputText: JSON.stringify(result.map),
        outputPath: mapFilePath
      });
    }

    return outputFiles;
  }
  /**
   * compile multiple components on the capsules
   */


  build(context) {
    var _this = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const capsules = context.capsuleGraph.seedersCapsules;
      const componentsResults = [];

      _this.setConfigFileFalse();

      const longProcessLogger = _this.logger.createLongProcessLogger('compile babel components', capsules.length);

      yield (0, _bluebird().mapSeries)(capsules, /*#__PURE__*/function () {
        var _ref = (0, _asyncToGenerator2().default)(function* (capsule) {
          const currentComponentResult = {
            errors: [],
            component: capsule.component
          };
          longProcessLogger.logProgress(capsule.component.id.toString());
          yield _this.buildOneCapsule(capsule, currentComponentResult);
          componentsResults.push(_objectSpread({}, currentComponentResult));
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }());
      return {
        artifacts: _this.getArtifactDefinition(),
        componentsResults
      };
    })();
  }

  createTask(name = 'BabelCompiler') {
    return this.compiler.createTask(name, this);
  }

  buildOneCapsule(capsule, componentResult) {
    var _this2 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      componentResult.startTime = Date.now();
      const sourceFiles = capsule.component.filesystem.files.map(file => file.relative);
      yield _fsExtra().default.ensureDir(_path().default.join(capsule.path, _this2.distDir));
      yield Promise.all(sourceFiles.map( /*#__PURE__*/function () {
        var _ref2 = (0, _asyncToGenerator2().default)(function* (filePath) {
          if (!_this2.isFileSupported(filePath)) {
            return;
          }

          let result;

          const absoluteFile = _path().default.join(capsule.path, filePath);

          try {
            result = yield babel().transformFileAsync(absoluteFile, _this2.options.babelTransformOptions);
          } catch (err) {
            var _componentResult$erro;

            (_componentResult$erro = componentResult.errors) === null || _componentResult$erro === void 0 ? void 0 : _componentResult$erro.push(err);
          }

          if (!result || !result.code) {
            return;
          }

          const distPath = _this2.replaceFileExtToJs(filePath);

          const distPathMap = `${distPath}.map`;
          const code = result.code || '';
          const outputText = result.map ? `${code}\n\n//# sourceMappingURL=${distPathMap}` : code;
          yield _fsExtra().default.outputFile(_path().default.join(capsule.path, _this2.distDir, distPath), outputText);

          if (result.map) {
            yield _fsExtra().default.outputFile(_path().default.join(capsule.path, _this2.distDir, distPathMap), JSON.stringify(result.map));
          }
        });

        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      }()));
      componentResult.endTime = Date.now();
    })();
  }
  /**
   * if it's not false, it searches for config files, which is probably not the expected behavior
   * here as the configuration is passed programmatically.
   * practically, when the configFile is not set, babel returns `null` for all files in the capsule
   */


  setConfigFileFalse() {
    this.options.babelTransformOptions = this.options.babelTransformOptions || {};
    this.options.babelTransformOptions.configFile = this.options.babelTransformOptions.configFile || false;
    this.options.babelTransformOptions.babelrc = this.options.babelTransformOptions.babelrc || false;
  }

  getArtifactDefinition() {
    return [{
      generatedBy: this.id,
      name: this.artifactName,
      globPatterns: this.distGlobPatterns
    }];
  }
  /**
   * given a source file, return its parallel in the dists. e.g. index.ts => dist/index.js
   */


  getDistPathBySrcPath(srcPath) {
    const fileWithJSExtIfNeeded = this.replaceFileExtToJs(srcPath);
    return _path().default.join(this.distDir, fileWithJSExtIfNeeded);
  }
  /**
   * whether babel is able to compile the given path
   */


  isFileSupported(filePath) {
    return (filePath.endsWith('.ts') || filePath.endsWith('.tsx') || filePath.endsWith('.js') || filePath.endsWith('.jsx')) && !filePath.endsWith('.d.ts');
  }

  displayConfig() {
    return JSON.stringify(this.options.babelTransformOptions || {}, null, 4);
  }

  replaceFileExtToJs(filePath) {
    if (!this.isFileSupported(filePath)) return filePath;

    const fileExtension = _path().default.extname(filePath);

    return filePath.replace(new RegExp(`${fileExtension}$`), '.js'); // makes sure it's the last occurrence
  }

}

exports.BabelCompiler = BabelCompiler;

//# sourceMappingURL=babel.compiler.js.map